package sim.application.util;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;

import javax.swing.JPanel;

/**
 * La classe <b>SJPanelImage</b> reprsente le panel o une image sera affiche.
 * 
 * @author Simon Vzina
 * @since 2015-08-22
 * @version 2020-03-23
 */
public class SJPanelImage extends JPanel {

  /**
   * 
   */
  private static final long serialVersionUID = -8117857790526440072L;
  
  /**
   * ...
   */
  private final static boolean DEFAULT_INTERPOLATION = false;
  
  /**
   * La constante <b>DEFAULT_IMAGE</b> reprsente l'image par dfaut affiche dans le JPanel.
   */
  private final static BufferedImage DEFAULT_IMAGE = new BufferedImage(500, 500, BufferedImage.TYPE_INT_RGB);
  
  /**
   * ...
   */
  private final boolean interpolation;
  
  /**
   * La variable <b>image</b> reprsente l'image  afficher dans le panel.
   */
  private BufferedImage image;
  
  //---------------
  // CONSTRUTEUR //
  //---------------
  
  /**
   * Constructeur d'un panel  image vide. L'image par dfaut sera null.
   */
  public SJPanelImage()
  {
    this(DEFAULT_INTERPOLATION);
  }

  /**
   * ...
   * 
   * @param interpolation
   */
  public SJPanelImage(boolean interpolation)
  {
    this(DEFAULT_IMAGE, interpolation);
  }
  
  /**
   * ...
   * 
   * @param image
   * @param interpolation
   */
  public SJPanelImage(BufferedImage image, boolean interpolation)
  {
    this.image = image;
    this.interpolation = interpolation;
  }
  
  //------------
  // MTHODES //
  //------------
  
  /**
   * Mthode pour dfinir l'image de type BufferedImage  assigner au panel.
   * @param image - L'image  dessiner.
   */
  public void setBufferedImage(BufferedImage image)
  {
    this.image = image;
    
    // Refaire l'affichage du JPanel, car le contenu a chang.
    repaint();
  }
  
  @Override
  public void paintComponent(Graphics g) 
  { 
    // Create new (blank) image of required (scaled) size
    //BufferedImage scaledImage = new BufferedImage(this.getWidth(), this.getHeight(), BufferedImage.TYPE_INT_ARGB);
    BufferedImage scaledImage = new BufferedImage(this.getWidth(), this.getHeight(), BufferedImage.TYPE_INT_RGB);
    
    // Paint scaled version of image to new image
    Graphics2D graphics2D = scaledImage.createGraphics();
      
    // Dfinir si l'image sera dessin avec ou sans interpolation des couleurs.
    if(interpolation)
      graphics2D.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR); 
            
    // Transposer l'image.
    graphics2D.drawImage(this.image, 0, 0, this.getWidth(), this.getHeight(), null);
      
    // clean up
    graphics2D.dispose();
      
    //Dessiner l'image redimensionne
    g.drawImage(scaledImage, 0, 0, null); 
  } 

}
