/**
 * 
 */
package sim.geometry;

import sim.exception.SNoImplementationException;
import sim.exception.SRuntimeException;
import sim.math.SMath;
import sim.math.SVector3d;

/**
 * La classe <b>SGeometricIntersection</b> reprsente une classe utilitaire permettant d'valuer des intersections
 * entre un rayon et diffrentes gomtries. 
 * <p>Le rayon tant paramtris dans le temps, la solution au intersection est
 * le temps requis afin qu'un rayon intersection la gomtrie. Un ensemble solution vide signifie qu'aucune intersection
 * n'a t ralis entre le rayon et la gomtrie et un ensemble solution multiple signifie que le rayon peut intersecter
 * la gomtrie  plusieurs endroits (dont plusieurs temps).</p>
 * 
 * @author Simon Vzina
 * @since 2015-11-08
 * @version 2017-12-20 (version labo  Le ray tracer v2.1)
 */
public class SGeometricIntersection {

  /**
   * Mthode permettant d'valuer l'intersection entre un rayon et un plan.
   * Un rayon peut raliser jusqu' <b>une intersection</b> avec un plan.
   * 
   * @param ray - Le rayon  intersecter avec le plan.
   * @param r_plane - La position de rfrence du plan.
   * @param n_plane - La normale  la surface du plan.
   * @return L'ensemble solution des temps pour raliser l'intersection entre le rayon et le plan <u>dans un tableau tri</u>. L'ensemble solution contient 0 ou 1 lment.
   */
  public static double[] planeIntersection(SRay ray, SVector3d r_plane, SVector3d n_plane)
  {
    throw new SNoImplementationException("Erreur SGeometricIntersection : La mthode n'a pas t implmente.");
  }
  
  /**
   * Mthode permettant d'valuer l'intersection entre un rayon et une sphre. 
   * Un rayon peut raliser jusqu' <b>deux intersections</b> avec une sphre.
   * 
   * @param ray - Le rayon  intersecter avec la sphre.
   * @param r_sphere - La position de la sphre.
   * @param R - Le rayon de la sphre (doit tre positif).
   * @return L'ensemble solution des temps pour raliser l'intersection entre le rayon et la sphre <u>dans un tableau tri</u>. L'ensemble solution contient 0, 1 ou 2 lments.
   */
  public static double[] sphereIntersection(SRay ray, SVector3d r_sphere, double R)
  {
    throw new SNoImplementationException("Erreur SGeometricIntersection : La mthode n'a pas t implmente.");
  }

  /**
   * Mthode permettant d'valuer l'intersection entre un rayon et un tube infini. 
   * Un rayon peut raliser jusqu' <b>deux intersections</b> avec le tube infini.
   *  
   * @param ray - Le rayon  intersecter avec le tube infini.
   * @param r_tube - Une position sur l'axe central du tube infini.
   * @param axis - L'axe du tube (doit tre normalis).
   * @param R - Le rayon du tube (doit tre positif).
   * @return L'ensemble solution des temps pour raliser l'intersection entre le rayon et le tube infini <u>dans un tableau tri</u>. L'ensemble solution contient 0, 1 ou 2 lments.
   */
  public static double[] infiniteTubeIntersection(SRay ray, SVector3d r_tube, SVector3d axis, double R)
  {
    throw new SNoImplementationException("Erreur SGeometricIntersection : La mthode n'a pas t implmente.");
  } 
  
  /**
   * Mthode permettant d'valuer l'intersection entre un rayon et deux cnes infinis reli par leur pointe. 
   * Un rayon peut raliser jusqu' <b>deux intersections</b> avec les deux cnes infinis.
   *  
   * @param ray - Le rayon  intersecter avec les deux cnes.
   * @param r_cone - Une position sur l'axe central des deux cnes o le rayon <i>R</i> a t dfini.
   * @param axis - L'axe des deux cnes dans la direction localisant la pointe du cne  partir de la position <i>r_cone</i> (doit tre normalis).
   * @param R - Le rayon du cne  la position <i>r_cone</i> (doit tre positif). 
   * @param H - La hauteur du cne tant dfinie comme la distance entre la position <i>r_cone</i> et la pointe des cnes (doit tre positif).
   * @return L'ensemble solution des temps pour raliser l'intersectin entre le rayon et les deux cnes infinis <u>dans un tableau tri</u>. L'ensemble solution contient 0, 1 ou 2 lments.
   */
  public static double[] infiniteTwoConeIntersection(SRay ray, SVector3d r_cone, SVector3d axis, double R, double H)
  {
    throw new SNoImplementationException("Erreur SGeometricIntersection : La mthode n'a pas t implmente.");
  }
  
  /**
   * Mthode permettant d'valuer l'intersection entre un rayon et un tore (beigne).
   * Un rayon peut raliser jusqu' <b>quatre intersections</b> avec un tore.
   * 
   * @param ray Le rayon  intersection avec le tore.
   * @param r_torus La position centrale du tore.
   * @param n_torus La normale au plan du tore.
   * @param R Le rayon de rvolution du cylindre formant le tore (doit tre positif).
   * @param r Le rayon de la partie cylindrique du tore (doit tre positif) 
   * @return L'ensemble solution des temps pour raliser l'intersection entre le rayon et le tore <u>dans un tableau tri</u>. L'ensemble solution contient 0, 1, 2, 3 ou 4 lments.
   * @throws SRuntimeException Si R ou r est ngatif.
   */
  public static double[] torusIntersection(SRay ray, SVector3d r_torus, SVector3d n_torus, double R, double r) throws SRuntimeException
  {
    if(R < 0)
      throw new SRuntimeException("Erreur SGeometricIntersection 001 : Le rayon du tore R = " + R + " est ngatif.");
    
    if(r < 0)
      throw new SRuntimeException("Erreur SGeometricIntersection 002 : Le rayon du cylindre du tore r = " + r + " est ngatif.");
    
    throw new SNoImplementationException("Erreur SGeometricIntersection : La mthode n'a pas t implmente.");
  }
  
}//fin de la classe SGeometricIntersection
