/**
 * 
 */
package sim.geometry.space;

import java.util.List;

import sim.exception.SRuntimeException;
import sim.geometry.SGeometry;
import sim.geometry.SRay;
import sim.math.SVector3d;
import sim.util.SLog;

/**
 * Classe qui reprsente un espace  gomtrie linaire. Cela signifie que les gomtries ne sont pas organises
 * dans une structure complexe. Il sera test en squence linaire  chaque tentative d'intersection avec un rayon.
 * Cette version trs nave est efficace seulement pour des scnes comportant un nombre trs trs faible de gomtries.
 * 
 * @author Simon Vzina
 * @since 2015-01-10
 * @version 2015-12-27
 */
public class SLinearSpace extends SAbstractGeometrySpace {

	/**
	 * Constructeur d'un espace  gomtrie linaire.
	 */
	public SLinearSpace()
	{
	  super();
	}
	
	@Override
	public SRay nearestIntersection(SRay ray, double t_max)throws SRuntimeException
	{
		//Vrifier si le rayon a dj intersect une gomtrie
		if(ray.asIntersected())
			throw new SRuntimeException("Erreur SLinearSpace 001 : Le rayon en paramtre a dj intersect une gomtrie.");
				
		//Vrifier la valeur de t_max est adquate
		if(t_max < 0.0)
			throw new SRuntimeException("Erreur SLinearSpace 002 : Le temps/distance maximale ne peut pas tre ngative.");
				
		// Vrifier si l'espace a t initialis
    if(!space_initialized)
      throw new SRuntimeException("Erreur SLinearSpace 003 : L'espace de gomtrie n'a pas t initialis.");
    
		//Obtenir la liste des intersections
		List<SRay> list_intersection = intersections(geometry_list, ray, t_max);
		
		//Si la liste n'est pas vide, la premire intersection sera l'lment [0] de la liste, 
		//car elle doit tre trie en ordre croissant de temps/distante
		if(!list_intersection.isEmpty())
			return list_intersection.get(0);
		else
			return ray;	//sans intersection, on retourne le rayon d'origine
	}
	
	@Override
	public List<SRay> nearestOpaqueIntersection(SRay ray, final double t_max)throws SRuntimeException
	{
	  //Vrifier si le rayon a dj intersect une gomtrie
    if(ray.asIntersected())
      throw new SRuntimeException("Erreur SLinearSpace 004 : Le rayon en paramtre a dj intersect une gomtrie.");
        
    //Vrifier la valeur de t_max est adquate
    if(t_max < 0.0)
      throw new SRuntimeException("Erreur SLinearSpace 005 : Le temps/distance maximale ne peut pas tre ngative.");
        
    // Vrifier si l'espace a t initialis
    if(!space_initialized)
      throw new SRuntimeException("Erreur SLinearSpace 006 : L'espace de gomtrie n'a pas t initialis.");
    
	  return nearestOpaqueIntersection(geometry_list, ray, t_max);
	}
	
  @Override
  public List<SGeometry> listInsideGeometry(SVector3d v) throws SRuntimeException
  {
    // Vrifier si l'espace a t initialis
    if(!space_initialized)
      throw new SRuntimeException("Erreur SLinearSpace 007 : L'espace de gomtrie n'a pas t initialis.");
    
    return listInsideGeometry(geometry_list, v);
  }
 
  @Override
  public void initialize()
  {
    SLog.logWriteLine("Message SLinearSpace : Construction de l'espace linaire des gomtries.");
    
    SLog.logWriteLine("Message SLinearSpace : Fin de la construction de l'espace linaire des gomtries.");
    SLog.logWriteLine();
    
    space_initialized = true;
  }
  
}//fin classe SLinearSpace
