/**
 * 
 */
package sim.geometry.space;

import sim.exception.SRuntimeException;

/**
 * La classe <b>SVoxelSpacePriorityEntry</b> reprsente une cellule de donnes pour un <b>SVoxelSpace</b> en comprenant un
 * itrateur FTVA (Fast Traversal Voxel Algorithm) sur les voxels particulier pour le test de l'intersection d'un rayon pralablement 
 * dtermin avec les gomtries de l'espace des voxels. C'est l'itrateur FTVA qui sera utilis dans l'attribution de la priorit 
 * l'aide de la mthode <i>public double nextMinTime()<i>.
 * 
 * @author Simon Vzina
 * @since 2016-02-04
 * @version 2016-02-04
 */
public class SVoxelSpacePriorityEntry implements Comparable<SVoxelSpacePriorityEntry>{

  /**
   * La variable <b>data</b> reprsente une cellule donnant accs aux donnes de l'espace des voxels (map, constructeur de voxel et le voxel d'extrme).
   */
  private final SVoxelSpaceEntry data;
  
  /**
   * La variable <b>FTVA</b> correspond  l'algorithme de parcourt de la carte de voxel (FastTraversalVoxelAlgorithm).
   * Cette variable doit tre redfinie  chaque fois qu'un nouveau rayon dsire intersecter des gomtries de la carte des voxels.
   */
  private final SFastTraversalVoxelAlgorithm FTVA;
  
  public SVoxelSpacePriorityEntry(SVoxelSpaceEntry data, SFastTraversalVoxelAlgorithm FTVA)
  {
    this.data = data;
    this.FTVA = FTVA;
  }

  //------------
  // MTHODES //
  //------------
  
  /**
   * Mthode pour obtenir les donnes dcrivant l'espace de voxels de la cellule.
   * 
   * @return Les donnes de l'espace de voxels
   */
  public SVoxelSpaceEntry getData()
  {
    return data;
  }
  
  /**
   * Mthode pour obtenir le FTVA (Fast Traversal Voxel Algorithm).
   * 
   * @return Le FTVA.
   */
  public SFastTraversalVoxelAlgorithm getFastTraversalVoxelAlgorithm() throws SRuntimeException
  {
    return FTVA;
  }
  
  @Override
  public int compareTo(SVoxelSpacePriorityEntry entry)
  {
    // La comparaison de cette classe se fera par le temps o un FTVA est rendu  itrer.
    // Cette information sera obtenue par la mthode nextMinTime() d'un objet FTVA tant le temps pour entrer dans le prochain voxel courant.
    // Ceci permettra d'ordonner en ordre de priorit les voxels des FTVA  visiter en premier tant de plus petit temps.
       
    return Double.compare(this.FTVA.nextMinTime(), entry.FTVA.nextMinTime());
  }
  
}//fin de la classe SVoxelSpacePriorityEntry
