/**
 * 
 */
package sim.graphics.shader;

import java.util.List;

import sim.exception.SNoImplementationException;
import sim.graphics.SColor;
import sim.graphics.material.SMaterial;
import sim.math.SVector3d;
import sim.physics.SGeometricalOptics;

/**
 * La classe <b>SIllumination</b> reprsente une classe utilitaire qui effectue les calculs d'illuminations selon le type d'algorithme utilis. 
 * On y retrouve des modles d'illumination  rflexion (ambiante, diffuse et spculaire) et  transmission (filtrage
 * d'une source de lumire au travers des matriaux transparents). 
 * 
 * @author Simon Vzina
 * @since 2015-01-16
 * @version 2017-12-20 (version labo  Le ray tracer v2.1)
 */
public class SIllumination {

  //--------------
  // CONSTANTES //
  //--------------
  
  /**
   * La constante <b>NO_ILLUMINATION</b> correspond  la couleur par dfaut sans calcul d'illumination.
   * Cette couleur est <b>noire</b>.
   */
	public static final SColor NO_ILLUMINATION = new SColor(0.0, 0.0, 0.0);
	
	//------------
	// MTHODES //
	//------------
	
	/**
	 * <p>Mthode qui effectue le calcul de la <u>rflexion ambiante</u> d'une source de lumire <b><i>L</i>a</b> sur une surface
	 * dont la couleur rflchie de faon ambiante est <b><i>S</i>a</b>.</p> 
	 * <p>La formule calcule est <b><i>L</i>amb = <i>L</i>a*<i>S</i>a</b>.</p> 
	 * 
	 * @param La - La couleur ambiante <b><i>L</i>a</b> de la source de lumire.
	 * @param Sa - La couleur rflchie de faon ambiante <b><i>S</i>a</b> par la surface.
	 * @return La couleur ambient <b><i>L</i>amb</b> de la lumire rflchie par la surface.
	 */
	public static SColor ambientReflexion(SColor La, SColor Sa)
	{
		throw new SNoImplementationException("La mthode n'est pas implmente.");
	}
	
	/**
	 * <p>Mthode qui effectue le calcul de la <u>rflexion diffuse</u> selon le <b>modle de rfexion Lambertienne</b> d'une source de lumire <b><i>L</i>d</b> sur une surface
	 * dont la couleur rflchie de faon diffuse est <b><i>S</i>d</b></b>.</p> 
	 * <p>La formule calcule est <ul><b><i>L</i>dif = <i>L</i>d*<i>S</i>d*<i>N</i>dot<i>L</i></b></ul></p>
	 * <p>o <b><i>N</i></b> est la normale  la surface et <b><i>L</i></b> est l'orientation <u>inverse</u> de la source de lumire.</p>
	 * 
	 * @param Ld - La couleur diffuse <b><i>L</i>d</b> de la source de lumire.
	 * @param Sd - La couleur rflchie de faon diffuse <b><i>S</i>d</b> par la surface.
	 * @param N - La normale <b><i>N</i></b>  la surface (unitaire).
	 * @param d - L'orientation <b><i>d</i></b> de la source de lumire (unitaire).
	 * @return La couleur diffuse <b><i>L</i>dif</b> de la lumire rflchie par la surface.
	 */
	public static SColor lambertianReflexion(SColor Ld, SColor Sd, SVector3d N, SVector3d d)
	{
	  throw new SNoImplementationException("La mthode n'est pas implmente.");
	}
	
	/**
	 * <p>Mthode qui effectue le calcul de la <u>rflexion spculaire</u> selon le <b>modle spculaire de <i>Phong</i></b> d'une source de lumire <b><i>L</i>s</b> sur une surface
	 * dont la couleur rflchie de faon spculaire est <b><i>S</i>s</b></b>.</p> 
	 * <p>La formule calcule est <ul><b><i>L</i>spe = <i>L</i>s*<i>S</i>s*(<i>R</i>dot<i>E</i>)^<i>n</i></b></ul></p>
	 * <p>o <b><i>R</i></b> est l'orientation de la rflexion de la lumire, <b><i>E</i></b> est l'orientation vers l'oeil (l'inverse de l'orientation du rayon) et <b><i>n</i></b> est le niveau de brillance.</p>
	 * 
	 * @param Ls - La couleur spculaire <b><i>L</i>s</b> de la source de lumire.
	 * @param Ss - La couleur rflchie de faon spculaire <b><i>S</i>s</b> par la surface.
	 * @param N - La normale <b><i>N</i></b>  la surface (unitaire).
	 * @param v - L'orientation <b><i>v</i></b> du rayon  (unitaire).
	 * @param d - L'orientation <b><i>d</i></b> de la source de lumire (unitaire).
	 * @param n - Le niveau de brillance <b><i>n</i></b> ( quel point la surface est polie).
	 * @return La couleur spculaire <b><i>L</i>spe</b> de la lumire rflchie par la surface.  
	 */
	public static SColor phongSpecularReflexion(SColor Ls, SColor Ss, SVector3d N, SVector3d v, SVector3d d, double n)
	{
	  throw new SNoImplementationException("La mthode n'est pas implmente.");
	}
	
	/**
	 * <p>Mthode qui effectue le calcul de la <u>rflexion spculaire</u> selon le <b>modle spculaire de <i>Blinn</i></b> d'une source de lumire <b><i>L</i>s</b> sur une surface
	 * dont la couleur rflchie de faon spculaire est <b><i>S</i>s</b></b>.</p> 
	 * <p>La formule calcule est <ul><b><i>L</i>spe = <i>L</i>s*<i>S</i>s*(<i>N</i>dot<i>H</i>)^<i>n</i></b></ul></p>
	 * <p>o <b><i>N</i></b> est l'orientation de la normale  la surface, <b><i>H</i></b> est l'orientation du vecteur bisecteur entre <b><i>E</i></b> et <b><i>L</i></b> et <b><i>n</i></b> est le niveau de brillance.</p>
	 * 
	 * @param Ls - La couleur spculaire <b><i>L</i>s</b> de la source de lumire.
	 * @param Ss - La couleur rflchie de faon spculaire <b><i>S</i>s</b> par la surface.
	 * @param N - La normale <b><i>N</i></b>  la surface (unitaire).
	 * @param v - L'orientation <b><i>v</i></b> du rayon  (unitaire).
	 * @param d - L'orientation <b><i>d</i></b> de la source de lumire (unitaire).
	 * @param n - Le niveau de brillance <b><i>n</i></b> ( quel point la surface est polie).
	 * @return La couleur spculaire <b><i>L</i>spe</b> de la lumire rflchie par la surface.  
	 */
	public static SColor blinnSpecularReflexion(SColor Ls, SColor Ss, SVector3d N, SVector3d v, SVector3d d, double n)
	{
	  throw new SNoImplementationException("La mthode n'est pas implmente.");
	}
	
	/**
	 * Mthode pour valuer la couleur d'une source de lumire aprs son passage au travers plusieurs matriaux transparents.
	 * 
	 * @param transparent_material_list - La liste des matriaux transparents traverss par la lumire.
	 * @param light_color - La couleur de la source de lumire.
	 * @return La couleur de la source de la lumire filtr par son passage au travers les matriaux transparents.
	 */
	public static SColor filteredTransparencyLight(List<SMaterial> transparent_material_list, SColor light_color)
	{
		SColor filtered_light = light_color;	//couleur de la lumire filtre par la transparence des gomtries traverses
		
		//Nous allons utiliser la couleur diffuse pour dterminer la couleur pouvant traverser un matriel.
		//Les couleurs des matriaux vont jouer le rle de filtre laissant passer uniquement la couleur du matriel
		//et pondr par leur niveau de transparence.
		for(SMaterial m : transparent_material_list)
			filtered_light = filtered_light.multiply(m.transparencyColor());
		
		
		return filtered_light;
	}
	
}//fin de la classe SIllumination
